{% extends "notebook.html" %}

{% block header_buttons %}
   <!-- insert appmode loading screen as early as possible -->
  <style>
    {% include 'appmode_top.css' %}
  </style>
  <div id="appmode-loader">
    <img src="{{ base_url + "nbextensions/appmode/gears.svg" }}" title="App is loading...">
  </div>

  {% if show_edit_button %}
  <span>
     <button id="appmode-leave" class="btn btn-default btn-sm navbar-btn pull-right" title="Edit this app as notebook">Edit App</button>
  </span>
  {% endif %}

  <span>
     <img id="appmode-busy" src="{{ base_url + "nbextensions/appmode/gears.svg" }}" title="Kernel Busy">
  </span>

  {% if show_other_buttons %}
  {{super()}}
  {% endif %}

{% endblock header_buttons %}

{% block script %}
  {{super()}}
  <!-- overwrite some styles -->
  <style>
    {% include 'appmode_bottom.css' %}
  </style>
{% endblock %}